% The optimization problem of finding an ellipsoid in the intersection of two ellipsoid 
% whose traded-off projections have maximum trace.
% 
% S. Kaynama for HSCC'15 submission, 18-10/2014

clear all; %close all; clc;

ellipsoid;
global ellOptions;
ellOptions.plot2d_grid = 1000;

n = 2;

%% Example 1:

% A = -3 + (3-(-3))*rand(n,n); A = A*A'; A = A + [1 0; 0 2];
% Ed = ellipsoid([1;1], A);
Ed = ellipsoid([1;1], [4 -6.9; -6.9 16]);
umax = 2; umin = -1;
Eu = ellipsoid([1;1], [30^2 0; 0 ((umax-umin)/2)^2]);
umax = 2; umin = -1;
Eu = ellipsoid([1;1], [30^2 0; 0 ((umax-umin)/2)^2]);


E2 = ellIntersectionIA([Ed Eu]);

P1 = [eye(n-1,n-1) zeros(n-1,1)];
P2 = [zeros(1,n-1) 1];

E2px = ellIntersectionIAProjET([Ed Eu],P1);

figure
options.fill = 0;
options.width = 2;
count = 0;
for alpha = 0:.1:1,
    count = count+1;
    subplot(3,4,count), hold on
    E2pp = ellIntersectionIAProjET_tradeoff([Ed Eu],P1,P2,alpha);
    plot(Eu,'g', Ed,'g',options)
    plot(E2,'b', E2px,'k', E2pp,'r')
    ylabel('u'), xlabel('x')
    %axis([-1 3 -3 5])
    axis equal
    title(sprintf('%2.0f%% favoring u',alpha*100))
end

ax = axes('position',[0,0,1,1],'visible','off'); 
tx = text(0.44,0.98,'(blue: regular | black: proj_x | red: tradeoff)'); 
set(tx,'fontweight','bold');



%% Example 2:

Ed = ellipsoid([1;1], [4 -6.9; -6.9 306]);
theta = pi/20;
Eu = [cos(theta) sin(theta); -sin(theta) cos(theta)]*Ed;

E2 = ellIntersectionIA([Ed Eu]);
E2maxvol = ellintersection_ia([Ed Eu]);

P1 = [eye(n-1,n-1) zeros(n-1,1)];
P2 = [zeros(1,n-1) 1];

E2px = ellIntersectionIAProjET([Ed Eu],P1);


figure
options.fill = 0;
options.width = 2;
count = 0;
for alpha = 0:.1:1,
    clf;
    hold on
    E2pp = ellIntersectionIAProjET_tradeoff([Ed Eu],P1,P2,alpha);
    plot(Eu,'g', Ed,'g',options)
    plot(E2,'b', E2maxvol,'k', E2pp,'r')
    ylabel('u'), xlabel('x')
    %axis([-1 3 -3 5])
    axis equal
    title(sprintf('%2.0f%% favoring u',alpha*100))
    hold off
    
ax = axes('position',[0,0,1,1],'visible','off'); 
tx = text(0.44,0.98,'(blue: maxtrace | black: maxvol | red: tradeoff)'); 
set(tx,'fontweight','bold');
    drawnow 
    pause
end



