% tightViabilityKernelVisualize: generate a pretty plot
%
% This script runs tightViabilityKernel(), and then generates a pretty
% visualization which shades in the target.

% Copyright 2011 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2009/11/28
% Modified to the viability kernel example, 2012/09/28
% Modified to the tight kernel example, 2014/10/12

  % Generate the viability kernel.  These are the settings used in the
  % figure from the paper; they take a long time to run.  The default
  % settings of tightViabilityKernel generate less accurate results but
  % require only about 1/16 the execution time.
  max_steps = 3;
  delta_fixed = 2;
  delta_jittered = 0;
  accuracy = 'veryHigh'; 
  %accuracy = 'medium';
  input_samples = 41;
  %input_samples = 7;
  state_dx = 0.025;
  %state_dx = 0.05;
  [ g, kernels ] = tightViabilityKernel(max_steps, delta_fixed, delta_jittered, accuracy, input_samples, state_dx);
  
  % Get rid of the figure generated during computation.
  close;
  
  % Convert the viability kernels over various horizons into a single
  % stepped function.  Regions outside the constraint will be -1 and
  % regions in K_n for step n will have value n. We need to work our way
  % forwards through the set of finite horizon sampled data viability
  % kernels because each viability kernel at step n-1 contains the kernel
  % at step n.
  summary = -ones(g.shape);
  for i = 1 : max_steps + 1;
    summary(kernels{i} <= 0) = i-1;
  end
  
  % Create the plot.
  figure;
  contourf(g.xs{:}, summary, 0 : max_steps);
  title(sprintf('Sampled Data Viability Kernel \\delta = %3.1f, N = %d', delta_fixed, max_steps))
  grid on;
  axis equal;
  axis(g.axis);
  
  % Make the color scheme more amenable to B&W printing.
  grayscale = gray(max_steps + 6);
  colormap(grayscale(4:end-2, :));
  
  % Now demonstrate a safe trajectory which lies within the estimated reach tube.
  hold on;
  xs = 2 * [ 2.0; 1.0; 0.0;  0.0 ];
  ys = 2 * [ 2.0; 1.0; 0.0; -1.0 ];
  plot(xs, ys, 'b-o', 'LineWidth', 2);
  
  xlabel('x_1');  ylabel('x_2');
  